import json
import logging
import pkgutil
import re
from collections import defaultdict
from collections import namedtuple

import pycountry

logger = logging.getLogger("le_utils")
logger.setLevel(logging.INFO)


LTR_LANGUAGE = "ltr"
RTL_LANGUAGE = "rtl"
LANGUAGE_DIRECTIONS = (
    (LTR_LANGUAGE, "Left to Right"),
    (RTL_LANGUAGE, "Right to Left"),
)

RTL_LANG_CODES = [
    "ar",  # Arabic
    "arq",  # Algerian
    "dv",  # Divehi; Dhivehi; Maldivian
    "he",  # Hebrew (modern)
    "fa",  # Persian
    "ps",  # Pashto; Pushto
    "ur",  # Urdu
    "yi",  # Yiddish
]


class Language(
    namedtuple(
        "Language", ["native_name", "primary_code", "subcode", "name", "ka_name"]
    )
):
    @property
    def code(self):
        if self.subcode:
            return "{primary_code}-{subcode}".format(
                primary_code=self.primary_code, subcode=self.subcode
            )
        else:
            return self.primary_code

    @property
    def id(self):
        return self.code

    @property
    def first_native_name(self):
        """
        Return the first native name in the comma-seprated list of `native_name`.
        """
        return self.native_name.split(",")[0]


def _parse_out_iso_639_code(code):
    code_regex = r"(?P<primary_code>\w{2,3})(-(?P<subcode>\w{2,4}))?"

    match = re.match(code_regex, code)
    if match:
        return defaultdict(lambda: None, **match.groupdict())
    else:
        return None


def generate_list(constantlist):
    for code, lang in constantlist.items():
        values = _parse_out_iso_639_code(code)
        values.update(lang)

        # add a default value to ka_name
        if "ka_name" not in values:
            values["ka_name"] = None

        yield Language(**values)


def _initialize_language_list():
    langlist = json.loads(
        pkgutil.get_data("le_utils", "resources/languagelookup.json").decode("utf-8")
    )

    return generate_list(langlist)


def _iget(key, lookup_dict):
    """
    Case-insensitive search for `key` within keys of `lookup_dict`.
    """
    for k, v in lookup_dict.items():
        if k.lower() == key.lower():
            return v
    return None


LANGUAGELIST = list(_initialize_language_list())

_LANGLOOKUP = {l.code: l for l in LANGUAGELIST}


def getlang(code, default=None):
    """
    Try to lookup a Language object for `code` in internal representation defined
    in resources/languagelookup.json.
    Returns None if lookup by internal representation fails.
    """
    return _LANGLOOKUP.get(code) or None
    # should this be   return _LANGLOOKUP.get(code) or _LANGLOOKUP[default]  ???


# NOTES ON INTERNAL REPRESENTATION FOR LANGUAGE CODES
################################################################################
# The language code lookup table in `resources/languagelookup.json` is complex:
# 1.   "<code>":{
# 2.     "name":"EnglishName, variant; AlternativeName",
# 3.     "native_name":"NativeName, AlternativeNativeName",
#       }
#
# 1. <code> can be two letter code, three letter code, or <code>-<locale> string
# 2. `name` attr. contains ";"-separated list of alternative names for the language,
#     some names have a ","-separated variant, e.g.,  "Spanish, Spain"
# 3. `native_name` contains ","-separated list of strings for language name in that language
#
# The following lookup tables and lookup functions perform the necessary logic
# to deal with the ";" and "," separators used in `resources/languagelookup.json`
################################################################################

_LANGUAGE_NAME_LOOKUP = {l.name: l for l in LANGUAGELIST}

# Enrich _LANGUAGE_NAME_LOOKUP with aliases for list-names, and simplified names
new_items = {}
for lang_name, lang_obj in _LANGUAGE_NAME_LOOKUP.items():
    # Add language names that are separated by semicolons, e.g. "Catalan; Valencian"
    if ";" in lang_name:
        new_names = [n.strip() for n in lang_name.split(";")]
        for new_name in new_names:
            if new_name in _LANGUAGE_NAME_LOOKUP.keys() or new_name in new_items:
                logger.debug("Skip " + new_name + " because it already exisits")
            else:
                new_items[new_name] = lang_obj
    # Add base names without modifies in brackets or country/region after comma
    elif "(" in lang_name or "," in lang_name:
        simple_name = lang_name.split(",")[0]  # take part before comma
        simple_name = simple_name.split("(")[0].strip()  # and before any bracket
        if simple_name in _LANGUAGE_NAME_LOOKUP.keys() or simple_name in new_items:
            logger.debug("Skip " + simple_name + " because it already exisits")
        else:
            new_items[simple_name] = lang_obj
_LANGUAGE_NAME_LOOKUP.update(new_items)


def getlang_by_name(name):
    """
    Try to lookup a Language object by name, e.g. 'English', in internal language list.
    Returns None if lookup by language name fails in resources/languagelookup.json.
    """
    direct_match = _iget(name, _LANGUAGE_NAME_LOOKUP)
    if direct_match:
        return direct_match
    else:
        simple_name = name.split(",")[0]  # take part before comma
        simple_name = simple_name.split("(")[0].strip()  # and before any bracket
        return _LANGUAGE_NAME_LOOKUP.get(simple_name, None)


_LANGUAGE_NATIVE_NAME_LOOKUP = {l.native_name: l for l in LANGUAGELIST}

# Enrich _LANGUAGE_NATIVE_NAME_LOOKUP with aliases for list-like names
new_items = {}
for lang_native_name, lang_obj in _LANGUAGE_NATIVE_NAME_LOOKUP.items():
    # Add base native names without modifies in brackets or after comma
    if "," in lang_native_name:
        new_native_names = [n.strip() for n in lang_native_name.split(",")]
        for new_native_name in new_native_names:
            simple_native_name = new_native_name.split("(")[
                0
            ].strip()  # text before any bracket
            if (
                simple_native_name in _LANGUAGE_NATIVE_NAME_LOOKUP.keys()
                or new_native_name in new_items
            ):
                logger.debug(
                    "Skip " + simple_native_name + " because it already exisits"
                )
            else:
                new_items[simple_native_name] = lang_obj
    elif "(" in lang_native_name:
        simple_native_name = lang_native_name.split("(")[
            0
        ].strip()  # text before any bracket
        if (
            simple_native_name in _LANGUAGE_NATIVE_NAME_LOOKUP.keys()
            or simple_native_name in new_items
        ):
            logger.debug("Skip " + simple_native_name + " because it already exisits")
        else:
            new_items[simple_native_name] = lang_obj
_LANGUAGE_NATIVE_NAME_LOOKUP.update(new_items)


def getlang_by_native_name(native_name):
    """
    Try to lookup a Language object by native_name, e.g. 'English', in internal language list.
    Returns None if lookup by language name fails in resources/languagelookup.json.
    """
    direct_match = _iget(native_name, _LANGUAGE_NATIVE_NAME_LOOKUP)
    if direct_match:
        return direct_match
    else:
        simple_native_name = native_name.split(",")[0]  # take part before comma
        simple_native_name = simple_native_name.split("(")[
            0
        ].strip()  # and before any bracket
        return _LANGUAGE_NATIVE_NAME_LOOKUP.get(simple_native_name, None)


def getlang_by_alpha2(code):
    """
    Lookup a Language object for language code `code` based on these strategies:
      - Special case rules for Hebrew and Chinese Hans/Hant scripts
      - Using `alpha_2` lookup in `pycountry.languages` followed by lookup for a
        language with the same `name` in the internal representaion
    Returns `None` if no matching language is found.
    """
    # First try exact match to a language code in the internal representaion
    exact_match = getlang(code)
    if exact_match:
        return exact_match

    # Handle special cases for language codes returned by YouTube API
    if code == "iw":  # handle old Hebrew code 'iw' and return modern code 'he'
        return getlang("he")
    elif "zh-Hans" in code:  # use code `zh-CN` for all simplified Chinese
        return getlang("zh-CN")
    elif re.match("zh(.*)?-TW", code):  # matches all Taiwan chinese codes
        return getlang("zh-TW")
    elif re.match("zh(.*)?-HK", code):  # use code `zh-Hant` for Hong Kong
        return getlang("zh-Hant")

    # extract prefix only if specified with subcode: e.g. zh-Hans --> zh
    first_part = code.split("-")[0]

    # See if pycountry can find this language
    try:
        pyc_lang = pycountry.languages.get(alpha_2=first_part)
        if pyc_lang:
            if hasattr(pyc_lang, "inverted_name"):
                lang_name = pyc_lang.inverted_name
            else:
                lang_name = pyc_lang.name
            return getlang_by_name(lang_name)
        else:
            return None
    except KeyError:
        return None


def getlang_direction(code):
    if code in RTL_LANG_CODES:
        return RTL_LANGUAGE
    else:
        return LTR_LANGUAGE



from le_utils.constants import file_formats

""" File Type Constants """
# constants for different File types
VIDEO = "video"
AUDIO = "audio"
DOCUMENT = "document"
EPUB = "epub"
HTML5 = "html5"
H5P = "h5p"
THUMBNAIL = "thumbnail"
SUBTITLES = "subtitles"
SLIDESHOW_IMAGE = "slideshow_image"


""" File Format (extension) to File Type Mapping """
MAPPING = {
    # ThumbnailFile formats
    file_formats.JPG: THUMBNAIL,
    file_formats.JPEG: THUMBNAIL,
    file_formats.PNG: THUMBNAIL,
    file_formats.GIF: THUMBNAIL,
    #
    # AudioFile formats
    file_formats.MP3: AUDIO,
    #
    # DocumentFile formats
    file_formats.PDF: DOCUMENT,
    file_formats.EPUB: EPUB,
    #
    # formats HTMLZipFile
    file_formats.HTML5: HTML5,
    # formats H5PFile
    file_formats.H5P: H5P,
    #
    # formats VideoFile
    file_formats.MP4: VIDEO,
    file_formats.WEBM: VIDEO,
    #
    # formats SubtitleFile
    file_formats.VTT: SUBTITLES,
}



import json
import pkgutil
from collections import namedtuple

""" License Constants """

# IMPORTANT: Keep these constants in sync with licenselookup.json

CC_BY = "CC BY"
CC_BY_SA = "CC BY-SA"
CC_BY_ND = "CC BY-ND"
CC_BY_NC = "CC BY-NC"
CC_BY_NC_SA = "CC BY-NC-SA"
CC_BY_NC_ND = "CC BY-NC-ND"
ALL_RIGHTS_RESERVED = "All Rights Reserved"
PUBLIC_DOMAIN = "Public Domain"
SPECIAL_PERMISSIONS = "Special Permissions"

choices = (
    (CC_BY, "CC BY"),
    (CC_BY_SA, "CC BY-SA"),
    (CC_BY_ND, "CC BY-ND"),
    (CC_BY_NC, "CC BY-NC"),
    (CC_BY_NC_SA, "CC BY-NC-SA"),
    (CC_BY_NC_ND, "CC BY-NC-ND"),
    (ALL_RIGHTS_RESERVED, "All Rights Reserved"),
    (PUBLIC_DOMAIN, "Public Domain"),
    (SPECIAL_PERMISSIONS, "Special Permissions"),
)


class License(
    namedtuple(
        "License",
        [
            "id",
            "name",
            "exists",
            "url",
            "description",
            "custom",
            "copyright_holder_required",
        ],
    )
):
    pass


def generate_list(constantlist):
    for id, lang in constantlist.items():
        yield License(id=int(id), **lang)


def _initialize_license_list():
    constantlist = json.loads(
        pkgutil.get_data("le_utils", "resources/licenselookup.json").decode("utf-8")
    )

    return generate_list(constantlist)


LICENSELIST = list(_initialize_license_list())



import json
import pkgutil
from collections import namedtuple

""" File Format Constants """

# IMPORTANT: Keep these constants in sync with formatlookup.json and presetlookup.json

# constants for Video format
MP4 = "mp4"
MP4_MIMETYPE = "video/mp4"
WEBM = "webm"
WEBM_MIMETYPE = "video/webm"
# constants for video formats converitble to mp4
AVI = "avi"
MOV = "mov"
MPG = "mpg"
WMV = "wmv"
MKV = "mkv"
FLV = "flv"
OGV = "ogv"
M4V = "m4v"

# constants for Subtitle format
VTT = "vtt"
VTT_MIMETYPE = ".vtt"
# constants for formats convertible to VTT
SRT = "srt"
TTML = "ttml"
SAMI = "sami"
SCC = "scc"
DFXP = "dfxp"

# constants for Audio format
MP3 = "mp3"
MP3_MIMETYPE = ".mp3"

# constants for Document format
PDF = "pdf"
PDF_MIMETYPE = "application/pdf"

# constants for Thumbnail format
JPG = "jpg"
JPG_MIMETYPE = "image/jpeg"
JPEG = "jpeg"
PNG = "png"
PNG_MIMETYPE = "image/png"
GIF = "gif"
GIF_MIMETYPE = "image/gif"


JSON = "json"
JSON_MIMETYPE = "application/json"
SVG = "svg"
SVG_MIMETYPE = "image/svg"
GRAPHIE = "graphie"
GRAPHIE_MIMETYPE = ".graphie"

# constants for Exercise format
PERSEUS = "perseus"
PERSEUS_MIMETYPE = "application/perseus+zip"

# constants for HTML5 zip format
HTML5 = "zip"
HTML5_MIMETYPE = ".zip"

# constants for H5P format
H5P = "h5p"
H5P_MIMETYPE = ".h5p"

# constants for ePub format
EPUB = "epub"
EPUB_MIMETYPE = "application/epub+zip"

choices = (
    (MP4, "MP4 Video"),
    (WEBM, "WEBM Video"),
    (VTT, "VTT Subtitle"),
    (MP3, "MP3 Audio"),
    (PDF, "PDF Document"),
    (JPG, "JPG Image"),
    (JPEG, "JPEG Image"),
    (PNG, "PNG Image"),
    (GIF, "GIF Image"),
    (JSON, "JSON"),
    (SVG, "SVG Image"),
    (PERSEUS, "Perseus Exercise"),
    (GRAPHIE, "Graphie Exercise"),
    (HTML5, "HTML5 Zip"),
    (H5P, "H5P"),
    (EPUB, "ePub Document"),
)


class Format(namedtuple("Format", ["id", "mimetype"])):
    pass


def generate_list(constantlist):
    for id, format in constantlist.items():
        yield Format(id=id, **format)


def _initialize_format_list():
    constantlist = json.loads(
        pkgutil.get_data("le_utils", "resources/formatlookup.json").decode("utf-8")
    )
    return generate_list(constantlist)


FORMATLIST = list(_initialize_format_list())

_FORMATLOOKUP = {f.id: f for f in FORMATLIST}


def getformat(id, default=None):
    """
    Try to lookup a file format object for its `id` in internal representation defined
    in resources/formatlookup.json.
    Returns None if lookup by internal representation fails.
    """
    return _FORMATLOOKUP.get(id) or None



import json
import pkgutil
from collections import namedtuple

from le_utils.constants import file_formats

""" Content Kind Constants """

# IMPORTANT: Keep these constants in sync with kindlookup.json and presetlookup.json

TOPIC = "topic"
VIDEO = "video"
AUDIO = "audio"
EXERCISE = "exercise"
DOCUMENT = "document"
HTML5 = "html5"
SLIDESHOW = "slideshow"
H5P = "h5p"

choices = (
    (TOPIC, "Topic"),
    (VIDEO, "Video"),
    (AUDIO, "Audio"),
    (EXERCISE, "Exercise"),
    (DOCUMENT, "Document"),
    (HTML5, "HTML5 App"),
    (SLIDESHOW, "Slideshow"),
    (H5P, "H5P"),
)

""" File Format (extension) to Content Kind Mapping """
MAPPING = {
    file_formats.MP4: VIDEO,
    file_formats.WEBM: VIDEO,
    file_formats.MP3: AUDIO,
    file_formats.PDF: DOCUMENT,
    file_formats.EPUB: DOCUMENT,
    file_formats.PERSEUS: EXERCISE,
    file_formats.HTML5: HTML5,
    file_formats.H5P: H5P,
}


class Kind(namedtuple("Kind", ["id", "name"])):
    pass


def generate_list(constantlist):
    for id, kind in constantlist.items():
        yield Kind(id=id, **kind)


def _initialize_kind_list():
    constantlist = json.loads(
        pkgutil.get_data("le_utils", "resources/kindlookup.json").decode("utf-8")
    )

    return generate_list(constantlist)


KINDLIST = list(_initialize_kind_list())



import json
import pkgutil
from collections import namedtuple

""" Format Preset Constants"""

# IMPORTANT: Keep these constants in sync with presetlookup.json

VIDEO_HIGH_RES = "high_res_video"
VIDEO_HIGH_RES_READABLE = "High Resolution"
VIDEO_LOW_RES = "low_res_video"
VIDEO_LOW_RES_READABLE = "Low Resolution"
VIDEO_VECTOR = "vector_video"
VIDEO_VECTOR_READABLE = "Vectorized"
VIDEO_THUMBNAIL = "video_thumbnail"
VIDEO_THUMBNAIL_READABLE = "Thumbnail"
VIDEO_SUBTITLE = "video_subtitle"
VIDEO_SUBTITLE_READABLE = "Subtitle"
VIDEO_DEPENDENCY = "video_dependency"
VIDEO_DEPENDENCY_READABLE = "Video (dependency)"

AUDIO = "audio"
AUDIO_READABLE = "Audio"
AUDIO_THUMBNAIL = "audio_thumbnail"
AUDIO_THUMBNAIL_READABLE = "Thumbnail"
AUDIO_DEPENDENCY = "audio_dependency"
AUDIO_DEPENDENCY_READABLE = "audio (dependency)"

DOCUMENT = "document"
DOCUMENT_READABLE = (
    "Document"  # TODO(ivan): Change to "PDF Document"  str translations?
)
EPUB = "epub"
EPUB_READABLE = "ePub Document"
DOCUMENT_THUMBNAIL = "document_thumbnail"
DOCUMENT_THUMBNAIL_READABLE = "Thumbnail"

EXERCISE = "exercise"
EXERCISE_READABLE = "Exercise"
EXERCISE_THUMBNAIL = "exercise_thumbnail"
EXERCISE_THUMBNAIL_READABLE = "Thumbnail"
EXERCISE_IMAGE = "exercise_image"
EXERCISE_IMAGE_READABLE = "Exercise Image"
EXERCISE_GRAPHIE = "exercise_graphie"
EXERCISE_GRAPHIE_READABLE = "Exercise Graphie"

CHANNEL_THUMBNAIL = "channel_thumbnail"
CHANNEL_THUMBNAIL_READABLE = "Channel Thumbnail"
TOPIC_THUMBNAIL = "topic_thumbnail"
TOPIC_THUMBNAIL_READABLE = "Thumbnail"

HTML5_ZIP = "html5_zip"
HTML5_ZIP_READABLE = "HTML5 Zip"
HTML5_THUMBNAIL = "html5_thumbnail"
HTML5_THUMBNAIL_READABLE = "HTML5 Thumbnail"

HTML5_DEPENDENCY_ZIP = "html5_dependency"
HTML5_DEPENDENCY_ZIP_READABLE = "HTML5 Dependency (Zip format)"

H5P_ZIP = "h5p"
H5P_ZIP_READABLE = "H5P Zip"
H5P_THUMBNAIL = "h5p_thumbnail"
H5P_THUMBNAIL_READABLE = "H5P Thumbnail"

SLIDESHOW_IMAGE = "slideshow_image"
SLIDESHOW_IMAGE_READABLE = "Slideshow Image"

SLIDESHOW_THUMBNAIL = "slideshow_thumbnail"
SLIDESHOW_THUMBNAIL_READABLE = "Slideshow Thumbnail"

SLIDESHOW_MANIFEST = "slideshow_manifest"
SLIDESHOW_MANIFEST_READABLE = "Slideshow Manifest"


choices = (
    (VIDEO_HIGH_RES, VIDEO_HIGH_RES_READABLE),
    (VIDEO_LOW_RES, VIDEO_LOW_RES_READABLE),
    # (VIDEO_VECTOR, VIDEO_VECTOR_READABLE),  # doesn't exist yet so took out
    (VIDEO_THUMBNAIL, VIDEO_THUMBNAIL_READABLE),
    (VIDEO_SUBTITLE, VIDEO_SUBTITLE_READABLE),
    (VIDEO_DEPENDENCY, VIDEO_DEPENDENCY_READABLE),
    (AUDIO, AUDIO_READABLE),
    (AUDIO_THUMBNAIL, AUDIO_THUMBNAIL_READABLE),
    (AUDIO_DEPENDENCY, AUDIO_DEPENDENCY_READABLE),
    (DOCUMENT, DOCUMENT_READABLE),
    (EPUB, EPUB_READABLE),
    (DOCUMENT_THUMBNAIL, DOCUMENT_THUMBNAIL_READABLE),
    (EXERCISE, EXERCISE_READABLE),
    (EXERCISE_THUMBNAIL, EXERCISE_THUMBNAIL_READABLE),
    (EXERCISE_IMAGE, EXERCISE_IMAGE_READABLE),
    (EXERCISE_GRAPHIE, EXERCISE_GRAPHIE_READABLE),
    (CHANNEL_THUMBNAIL, CHANNEL_THUMBNAIL_READABLE),
    (TOPIC_THUMBNAIL, TOPIC_THUMBNAIL_READABLE),
    (HTML5_ZIP, HTML5_ZIP_READABLE),
    (HTML5_DEPENDENCY_ZIP, HTML5_DEPENDENCY_ZIP_READABLE),
    (HTML5_THUMBNAIL, HTML5_THUMBNAIL_READABLE),
    (H5P_ZIP, H5P_ZIP_READABLE),
    (H5P_THUMBNAIL, H5P_THUMBNAIL_READABLE),
    (SLIDESHOW_IMAGE, SLIDESHOW_IMAGE_READABLE),
    (SLIDESHOW_THUMBNAIL, SLIDESHOW_THUMBNAIL_READABLE),
    (SLIDESHOW_MANIFEST, SLIDESHOW_MANIFEST_READABLE),
)


class Preset(
    namedtuple(
        "Preset",
        [
            "id",
            "readable_name",
            "multi_language",
            "supplementary",
            "thumbnail",
            "subtitle",
            "display",
            "order",
            "kind",
            "allowed_formats",
            "convertible_formats",
        ],
    )
):
    pass


def generate_list(constantlist):
    for id, preset in constantlist.items():
        yield Preset(id=id, **preset)


def _initialize_preset_list():
    constantlist = json.loads(
        pkgutil.get_data("le_utils", "resources/presetlookup.json").decode("utf-8")
    )

    return generate_list(constantlist)


PRESETLIST = list(_initialize_preset_list())



"""
humanhash: Human-readable representations of digests.

The simplest ways to use this module are the :func:`humanize` and :func:`uuid`
functions. For tighter control over the output, see :class:`HumanHasher`.
"""
import uuid

# Copyright (c) 2014 SUNET. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright notice, this list of
#       conditions and the following disclaimer.
#
#    2. Redistributions in binary form must reproduce the above copyright notice, this list
#       of conditions and the following disclaimer in the documentation and/or other materials
#       provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY SUNET ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SUNET OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are those of the
# authors and should not be interpreted as representing official policies, either expressed
# or implied, of SUNET.
#
"""
Python implementation of proquints.
Proquints are PRO-nouncable QUINT-uplets of alternating unambiguous consonants
and vowels. See http://arxiv.org/html/0901.4016 for more info.
Copyright (c) 2014 SUNET. All rights reserved.
See the source file for complete license statement.
"""


__version__ = "0.1.0"
__copyright__ = "SUNET"
__organization__ = "SUNET"
__license__ = "BSD"
__authors__ = ["Fredrik Thulin"]

__all__ = []

CONSONANTS = "bdfghjklmnprstvz"
VOWELS = "aiou"


def _from_int(data):
    """
    :params data: integer
    :returns: proquint made from input data
    :type data: int
    :rtype: string
    """
    res = []
    while data > 0 or not res:
        for j in range(5):
            if not j % 2:
                res += CONSONANTS[(data & 0xF)]
                data >>= 4
            else:
                res += VOWELS[(data & 0x3)]
                data >>= 2
        if data > 0:
            res += "-"
    res.reverse()
    return "".join(res)


def generate():
    """
    :returns: proquint
    :rtype: int
    """
    return _from_int(int(uuid.uuid4().hex[:8], 16)).replace("-", "")



# coding=utf-8
""" Mastery Models """
DO_ALL = "do_all"
NUM_CORRECT_IN_A_ROW_10 = "num_correct_in_a_row_10"
NUM_CORRECT_IN_A_ROW_2 = "num_correct_in_a_row_2"
NUM_CORRECT_IN_A_ROW_3 = "num_correct_in_a_row_3"
NUM_CORRECT_IN_A_ROW_5 = "num_correct_in_a_row_5"
SKILL_CHECK = "skill_check"
M_OF_N = "m_of_n"

MASTERY_MODELS = (
    (DO_ALL, "Do all"),
    (NUM_CORRECT_IN_A_ROW_2, "2 in a row"),
    (NUM_CORRECT_IN_A_ROW_10, "10 in a row"),
    (NUM_CORRECT_IN_A_ROW_3, "3 in a row"),
    (NUM_CORRECT_IN_A_ROW_5, "5 in a row"),
    (SKILL_CHECK, "Skill check"),
    (M_OF_N, "M out of N"),
)

IMG_PLACEHOLDER = "{☣ LOCALPATH}"
IMG_FORMAT = "${" + IMG_PLACEHOLDER + "}/{0}"
IMG_REGEX = r"\$\{☣ LOCALPATH\}\/(.+)"

CONTENT_STORAGE_PLACEHOLDER = "{☣ CONTENTSTORAGE}"
CONTENT_STORAGE_FORMAT = "${" + CONTENT_STORAGE_PLACEHOLDER + "}/{0}"
CONTENT_STORAGE_REGEX = r"\(\$\{☣ CONTENTSTORAGE\}\/(.+)\)"

GRAPHIE_DELIMITER = "$$$GRAPHIE_BREAK$$$"

""" Question Types """
INPUT_QUESTION = "input_question"
MULTIPLE_SELECTION = "multiple_selection"
SINGLE_SELECTION = "single_selection"
FREE_RESPONSE = "free_response"
PERSEUS_QUESTION = "perseus_question"

question_choices = (
    (INPUT_QUESTION, "Input Question"),
    (MULTIPLE_SELECTION, "Multiple Selection"),
    (SINGLE_SELECTION, "Single Selection"),
    (FREE_RESPONSE, "Free Response"),
    (PERSEUS_QUESTION, "Perseus Question"),
)



import io

from setuptools import find_packages
from setuptools import setup

long_description = io.open("README.md", encoding="utf-8").read()

requirements = [
    "pycountry==17.5.14",
]

setup(
    name="le-utils",
    packages=find_packages(),
    version="0.1.28",
    description="LE-Utils contains shared constants used in Kolibri, Ricecooker, and Kolibri Studio.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    license="MIT",
    url="https://github.com/learningequality/le-utils",
    download_url="https://github.com/learningequality/le-utils/releases",
    keywords="le-utils le_utils LE utils kolibri studio ricecooker content curation",
    package_data={
        "le_utils": ["resources/*.json"],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Topic :: Utilities",
    ],
    author="Learning Equality",
    author_email="info@learningequality.org",
)
